# -*- coding: latin-1 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

import os.path
import codecs
from xml.sax import make_parser
from xml.sax.saxutils import quoteattr

from formed.meta.nodecomponents import RootNode, CaseNode, LogbookNode, Leaf, \
     Node
from formed.meta.document import Document
from formed.meta.data import PageNode, LogbookNode, GroupNode, RepeatNode, \
     MatrixNode, RadioNode, BoolLeaf, ChoiceNode, ExternalChoiceListLeaf, \
     IntLeaf, InfoLeaf, TextLeaf, TextAreaLeaf, RuleLeaf, DateLeaf, \
     ConditionalNode, NumericLeaf
from formed.meta.io.parser import SAXBuilder
from formed.meta.io.annotations import Annotations
from formed.meta.io.factories import DocumentFactory, SimpleCreator, \
     NodeComponentFactory, NodeFactory


DOCUMENT_TAG        = u"document"
CASE_TAG            = u"case"
LOGBOOK_TAG         = u"logbook"
PAGE_TAG            = u"page"
GROUP_TAG           = u"group"
MATRIX_TAG          = u"radio-matrix"
RADIO_TAG           = u"radio"
BOOL_TAG            = u"bool"
INT_TAG             = u"int"
NUMERIC_TAG         = u"float"
INFO_TAG            = u"info"
TEXT_TAG            = u"text"
CHOICE_TAG          = u"choice"
EXT_CHOICE_LIST_TAG = u"external-choice"
DATE_TAG            = u"date"
TEXT_AREA_TAG       = u"textarea"
RULE_TAG            = u"rule"
CONDITIONAL_TAG     = u"conditional"
REPEAT_TAG          = u"repeat"

namesToClasses = {
    DOCUMENT_TAG:        RootNode,
    CASE_TAG:            CaseNode,
    LOGBOOK_TAG:         LogbookNode,
    PAGE_TAG:            PageNode,
    GROUP_TAG:           GroupNode,
    MATRIX_TAG:          MatrixNode,
    RADIO_TAG:           RadioNode,
    BOOL_TAG:            BoolLeaf,
    CHOICE_TAG:          ChoiceNode,
    EXT_CHOICE_LIST_TAG: ExternalChoiceListLeaf,
    INT_TAG:             IntLeaf,
    NUMERIC_TAG:         NumericLeaf,
    INFO_TAG:            InfoLeaf,
    TEXT_TAG:            TextLeaf,
    TEXT_AREA_TAG:       TextAreaLeaf,
    RULE_TAG:            RuleLeaf,
    DATE_TAG:            DateLeaf,
    CONDITIONAL_TAG:     ConditionalNode,
    REPEAT_TAG:          RepeatNode
    }

factoryCreators = {
    DOCUMENT_TAG:        SimpleCreator(DocumentFactory,      RootNode),
    CASE_TAG:            SimpleCreator(NodeFactory,          CaseNode),
    LOGBOOK_TAG:         SimpleCreator(NodeFactory,          LogbookNode),
    PAGE_TAG:            SimpleCreator(NodeFactory,          PageNode),
    GROUP_TAG:           SimpleCreator(NodeFactory,          GroupNode),
    REPEAT_TAG:          SimpleCreator(NodeFactory,          RepeatNode),
    MATRIX_TAG:          SimpleCreator(NodeFactory,          MatrixNode),
    CONDITIONAL_TAG:     SimpleCreator(NodeFactory,          ConditionalNode),
    RADIO_TAG:           SimpleCreator(NodeFactory,          RadioNode),
    CHOICE_TAG:          SimpleCreator(NodeFactory,          ChoiceNode),
    EXT_CHOICE_LIST_TAG: SimpleCreator(NodeComponentFactory,
                                       ExternalChoiceListLeaf),
    BOOL_TAG:            SimpleCreator(NodeComponentFactory, BoolLeaf),
    INT_TAG:             SimpleCreator(NodeComponentFactory, IntLeaf),
    NUMERIC_TAG:         SimpleCreator(NodeComponentFactory, NumericLeaf),
    INFO_TAG:            SimpleCreator(NodeComponentFactory, InfoLeaf),
    TEXT_AREA_TAG:       SimpleCreator(NodeComponentFactory, TextAreaLeaf),
    TEXT_TAG:            SimpleCreator(NodeComponentFactory, TextLeaf),
    RULE_TAG:            SimpleCreator(NodeComponentFactory, RuleLeaf),
    DATE_TAG:            SimpleCreator(NodeComponentFactory, DateLeaf)}

classesToTagNames = dict([(v, k) for k, v in namesToClasses.iteritems()])

ANNOTATIONS_FILE = "annotations.xhtml"


def openDocument(path, documentFactoryCreator=None):
    # Load annotations first
    dirname = os.path.dirname(path)
    annotations_file = os.path.join(dirname, ANNOTATIONS_FILE)

    annotations = Annotations(annotations_file)

    builder = SAXBuilder(path)

    if documentFactoryCreator:
        factories = factoryCreators.copy()
        factories[DOCUMENT_TAG] = documentFactoryCreator
    else:
        factories = factoryCreators

    builder.setCreators(factories)

    f = None
    try:
        f = file(path, "rb")
        parser = make_parser()
        parser.setContentHandler(builder)
        parser.parse(f)

        document = builder.getDocument()
        document.init()
        annotations.decorate(document)

        return document
    finally:
        if f is not None:
            try:
                f.close()
            except:
                pass


def _attributesToXML(nc):
    out = []
    for key, value in nc.attributes.iteritems():
        if value:
            out.append(u" %s=%s" % (key, quoteattr(value)))
    return " ".join(out)


def _toXML(nc):
    if isinstance(nc, Node):
        tagName = classesToTagNames.get(nc.__class__, "node")
        out = [u"<%s%s>" % (tagName, _attributesToXML(nc))]
        for child in nc.children:
            out.append(_toXML(child))
        out.append(u"</%s>" % tagName)
        return "".join(out)
    if isinstance(nc, Leaf):
        tagName = classesToTagNames.get(nc.__class__, "leaf")
        return u"<%s%s/>" % (tagName, _attributesToXML(nc))
    if isinstance(nc, Document):
        out = [u'<?xml version="1.0" encoding="UTF-8"?>\n']
        out.append(_toXML(nc.root))
        return "".join(out)
    return ""


def saveDocument(document, path):
    f = None
    try:
        f = codecs.open(path, "wb", "UTF-8")
        print >> f, _toXML(document)
    finally:
        if f is not None:
            try:
                f.close()
            except:
                pass

# vim:set ts=4 sw=4 si et sta sts=4:
