# -*- coding: utf-8 -*-
from cgi import escape
import logging

import formencode
import paste

from pylons import app_globals as g

from formed.meta.data import PageNode

from mpulsweb.lib.translation import _, ungettext
from mpulsweb.lib.base import c, render, request, session, BaseController
from mpulsweb.lib.helpers import url_for
from mpulsweb.lib.security import checkRole
from mpulsweb.lib.validators import BundleActionForm, SetBundleEditorForm, \
    SetBundleStandinForm, ExportSelectorValidator
from mpulsweb.lib.helper.filters import NA
from mpulsweb.model.user import UserGroupList, UserListObject, UserObject
from mpulsweb.model.case import MpulsCaseBundle, MpulsCaseOverview
from mpulsweb.model.phase import ConsistenceCheckException
from mpulsweb.controllers.case_overview import parseSearchOptions, \
     search_options_from_session


log = logging.getLogger(__name__)


BUNDLE_NOTIFICATION_NOACTION_HEADER = _("No action specified for the selection")
BUNDLE_NOTIFICATION_NOACTION = _("""You have not selected any action for the
selection. Please click OK to go to the case overview and select there
an action from the drop down menu.""")

BUNDLE_NOTIFICATION_EMPTYLIST_HEADER = _("No valid cases in the selection!")
BUNDLE_NOTIFICATION_EMPTYLIST = _(u"""The selected action can not be executed
on any of the cases in your selection. Please make sure that the cases are
in a state from which you can execute the selected action.""")

OK_TO_CONTINUE = _("Please click OK to continue.")



class CaseBundleController(BaseController):

    def index(self):
        return "Hello world."

    def bundleAction(self):
        validator = BundleActionForm()
        log.debug(request.params)
        try:
            # Check submitted fields
            form_result = validator.to_python(request.params)
            log.debug(form_result)

            # Check if the user has selected any cases
            if (len(form_result.get('case_id', [])) <= 0
                and not form_result.get('all_cases')):
                return self._empty_caselist()

            # Build case bundle
            case_bundle = self._build_casebundle(form_result)

            # Delegate to concrete action
            action = form_result.get('action')
            # No action
            if action == 'no_action':
                return self.no_action()
            if action == 'markdelete':
                # Fiter out case which are not open(1) or inactive (2)
                case_bundle = self._filterBundle(case_bundle,
                                                 allowedstates=(1, 2))
                if case_bundle.isEmpty():
                    return self._empty_caselist()
                session['casebundle'] = case_bundle
                session.save()
                return self.markDelete(0)
            if action == 'delete':
                # Fiter out case which are not marked for deletion
                case_bundle = self._filterBundle(case_bundle,
                                                 allowedstates=(3,))
                if case_bundle.isEmpty():
                    return self._empty_caselist()
                session['casebundle'] = case_bundle
                session.save()
                return self.delete(0)
            if action == 'markanonymize':
                # Fiter out case which are not open(1) or inactive (2)
                # or not anonymizable
                case_bundle = self._filterBundle(case_bundle,
                                                 allowedstates=(1, 2),
                                                 filter_notanonymizable=True)
                if case_bundle.isEmpty():
                    return self._empty_caselist()
                session['casebundle'] = case_bundle
                session.save()
                return self.markAnonymize(0)
            if action == 'anonymize':
                # Fiter out case which are not open(1) or inactive (2)
                # or not anonymizable
                case_bundle = self._filterBundle(case_bundle,
                                                 allowedstates=(4,),
                                                 filter_notanonymizable=True)
                if case_bundle.isEmpty():
                    return self._empty_caselist()
                session['casebundle'] = case_bundle
                session.save()
                return self.anonymize(0)
            if action == 'restore':
                # Filter out cases which aren't marked for anonymization (4),
                # or marked for deletion (3)
                case_bundle = self._filterBundle(case_bundle,
                                                 allowedstates=(3, 4))
                if case_bundle.isEmpty():
                    return self._empty_caselist()
                session['casebundle'] = case_bundle
                session.save()
                return self.restore(0)
            if action == 'standin':
                # filter out cases where the user is not the editor and
                # therefor is not allowed to set the standin.
                user = session['USER_AUTHORIZED']
                filtered = []
                for case in case_bundle.getCases():
                    if (case.getEditor().id != user.id and not user.isAdmin()):
                        filtered.append(case.id)

                # Fiter out case which are not open(1) or inavtice (2)
                case_bundle = self._filterBundle(case_bundle, (1, 2), filtered)
                if case_bundle.isEmpty():
                    return self._empty_caselist()
                session['casebundle'] = case_bundle
                session.save()

                group_list = UserGroupList()
                user_list = UserListObject()
                group_filter = [u.gid for u in user_list.getAdminList()]
                group_filter.append(user.gid)
                c.group_list = group_list.getGroupList(group_filter)
                c.case_list = case_bundle.getCases()
                c.current_user = user

                return render('/casebundle/standin.mako')
            if action == 'transfer':
                # Fiter out case which are not open(1) or inavtice (2)
                user = session['USER_AUTHORIZED']
                case_bundle = self._filterBundle(case_bundle, (1, 2))
                if case_bundle.isEmpty():
                    return self._empty_caselist()
                session['casebundle'] = case_bundle
                session.save()

                user_list = UserListObject()
                user_filter = [u.id for u in  user_list.getAdminList()]
                user_filter.append(user.id)
                c.user_list = user_list.getUserList(user_filter)
                c.case_list = case_bundle.getCases()

                return render('/casebundle/editor.mako')

            if action.startswith("export"):
                # allow export of cases that are active (1), marked for
                # anonymization (4) or anonymized (5).  For historical
                # reasons, there's also the old now unused state 2 in
                # the filter tuple.
                case_bundle = self._filterBundle(case_bundle, (1, 2, 4, 5))
                if case_bundle.isEmpty():
                    return self._empty_caselist()
                session['casebundle'] = case_bundle
                session.save()
                export_format = action[6:]
                if export_format in ("XML", "CSV", "XLS"):
                    return self.export(export_format)

            if action == 'evaluate':
                case_bundle = self._filterBundle(case_bundle, (1, 4, 5))
                if case_bundle.isEmpty():
                    return self._emptyListError()

                session['casebundle'] = case_bundle
                session["evaluation_ids"] = case_bundle.listDatasetIds()
                session.save()
                c.url_ok = url_for(controller='/case_overview')
                num = len(case_bundle)
                c.dialog_title= ungettext("Case was marked for evaluation",
                                          "Cases were marked for evaluation",
                                          num)
                c.dialog_text = \
                    ungettext("%d case was marked for evaluation."
                              " All following evaluations will be limited"
                              " to that case. Click 'OK' to continue.",
                              "%d cases were marked for evaluation."
                              " All following evalutations will be limited"
                              " to those cases. Click 'OK' to continue.",
                              num) % num

                return render('/casebundle/dialogs/success_markevaluate.mako')

        except formencode.Invalid, error:
            log.exception(error)
            raise
        except Exception, e:
            log.exception(e)
            raise
        return "bundleAction"

    def _filterBundle(self, case_bundle, allowedstates, filtered_cases=None,
                      filter_notanonymizable=False):
        """Returns a filtered CaseBundle. Filtering can be done on
        states and on case_ids.
        """
        filtered = []
        for case in case_bundle.getCases(filtered_cases):
            if filter_notanonymizable:
                try:
                    case.check_anonymizeability()
                except ConsistenceCheckException, e:
                    continue
            state = case.getState()
            if state.getState() in allowedstates:
                filtered.append(case.id)
        case_bundle = MpulsCaseBundle(filtered)
        return case_bundle

    def _buildCaseList(self, bundle):
        list = ["<ul>"]
        for case in bundle.getCases():
            list.append("<li>%s, %s</li>" % (escape(NA(case.first_name)),
                                             escape(NA(case.last_name))))
        list.append("</ul>")
        return list

    def _build_casebundle(self, options):
        if options.get('all_cases') > 0:
            ids = options["all_ids"]
        else:
            ids = options.get("case_id")
        return MpulsCaseBundle(ids)

    def _empty_caselist(self):
        c.url_ok = url_for(controller='/case_overview')
        c.dialog_title = BUNDLE_NOTIFICATION_EMPTYLIST_HEADER
        c.dialog_text = BUNDLE_NOTIFICATION_EMPTYLIST
        return render('/casebundle/dialogs/error.mako')

    #
    # ACTIONS
    #
    def no_action(self):
        c.dialog_title= BUNDLE_NOTIFICATION_NOACTION_HEADER
        c.dialog_text = BUNDLE_NOTIFICATION_NOACTION
        c.url_ok = url_for(controller='/case_overview')
        return render('/casebundle/dialogs/error.mako')

    # DELETE
    @checkRole(('cm'))
    def markDelete(self, confirmed):
        confirmed = self._checkBool(confirmed)
        c.context = "../main.mako"
        case_bundle = session.get('casebundle')
        if confirmed == 1:
            num = case_bundle.markDelete()
            c.dialog_title = ungettext(u"Case deleted!", u"Cases deleted!", num)
            c.dialog_text = (ungettext(u"%s case was successfully cleared for"
                                       u" deletion.",
                                       u"%s cases were successfully cleared for"
                                       u" deletion.", num) % num
                             + " " + OK_TO_CONTINUE)
            c.url_ok = url_for(controller='/case_overview')
            del session['casebundle']
            return render ('/casebundle/dialogs/success_markdelete.mako')
        else:
            c.context = "../main.mako"
            c.dialog_title = ungettext(u'Delete case?', u'Delete cases?',
                                       len(case_bundle.getCases()))
            c.dialog_text = (ungettext('Do you really want to delete the'
                                       ' following case?%s',
                                       'Do you really want to delete the'
                                       ' following cases?%s',
                                       len(case_bundle.getCases()))
                             % "\n".join(self._buildCaseList(case_bundle)))
            c.url_yes = url_for(controller='/case_bundle', action='markDelete',
                                confirmed=1)
            c.url_no = url_for(controller='/case_overview')
            return render('/casemanagement/dialogs/confirm.mako')

    @checkRole(('admin'))
    def delete(self, confirmed):
        confirmed = self._checkBool(confirmed)
        c.context = "../main.mako"
        case_bundle = session.get('casebundle')
        if confirmed == 1:
            num = case_bundle.delete()
            c.dialog_title = ungettext(u"Case successfully deleted.",
                                       u"Cases successfully deleted.",
                                       num)
            c.dialog_text = (ungettext(u"%s case was successfully deleted.",
                                       u"%s cases were successfully deleted.",
                                       num) % num
                             + " " + OK_TO_CONTINUE)
            c.url_ok = url_for(controller='/case_overview')
            del session['casebundle']
            return render('/casebundle/dialogs/success_delete_admin.mako')
        else:
            c.dialog_title = ungettext(u'Delete case?', u'Delete cases?',
                                       len(case_bundle.getCases()))
            c.dialog_text = (ungettext('Do you really want to delete the'
                                       ' following case?%s',
                                       'Do you really want to delete the'
                                       ' following cases?%s',
                                       len(case_bundle.getCases()))
                             % "\n".join(self._buildCaseList(case_bundle)))
            c.url_yes = url_for(controller='/case_bundle', action='delete',
                                confirmed=1)
            c.url_no = url_for(controller='/case_overview')
            return render('/casemanagement/dialogs/confirm.mako')

     # ANONYMIZE
    @checkRole(('cm'))
    def markAnonymize(self, confirmed):
        confirmed = self._checkBool(confirmed)
        c.context = "../main.mako"
        case_bundle = session.get('casebundle')
        if confirmed == 1:
            num = case_bundle.markAnonymize()
            c.success_for = ungettext(u"Case successfully cleared for"
                                      u" anonymisation!",
                                      u"Cases successfully cleared for"
                                      u" anonymisation!",
                                      num)
            c.success_text = (
                ungettext(u"%s case was cleared for anonymisation. It will be"
                          u" presented to the administrators for review for"
                          u" the definite anonymisation.",
                          u"%s cases were cleared for anonymisation. They will"
                          u" be presented to the administrators for review for"
                          u" the definite anonymisation.", num) % num
                + " " + OK_TO_CONTINUE)
            c.url_ok = url_for(controller='/case_overview')
            del session['casebundle']
            return render ('/casebundle/dialogs/success_anonymize.mako')
        else:
            c.dialog_title = ungettext(u'Clear case for anonymisation?',
                                       u'Clear cases for anonymisation?',
                                       len(case_bundle.getCases()))
            c.dialog_text = (
                ungettext(u"Do you really want to clear the following case for"
                          u" anonymisation? The case will not be available to"
                          u" you to work on it and it will be presented to the"
                          u" administrators for review for the definite"
                          u" anonymisation.%s",
                          u"Do you really want to clear the following cases for"
                          u" anonymisation? The cases will not be available to"
                          u" you to work on them and they will be presented to"
                          u" the administrators for review for the definite"
                          u" anonymisation.%s",
                          len(case_bundle.getCases()))
                % "\n".join(self._buildCaseList(case_bundle)))
            c.url_yes = url_for(controller='/case_bundle',
                                action='markAnonymize', confirmed=1)
            c.url_no = url_for(controller='/case_overview')
            return render('/casebundle/dialogs/confirm_anonymize.mako')

    @checkRole(('admin'))
    def anonymize(self, confirmed):
        confirmed = self._checkBool(confirmed)
        c.context = "../main.mako"
        case_bundle = session.get('casebundle')
        if confirmed == 1:
            num = case_bundle.anonymize()
            c.dialog_title = ungettext(u"Case anonymised!",
                                       u"Cases anonymised!",
                                       num)
            c.dialog_text = (ungettext(u"%s case was successfully anonymised.",
                                       u"%s cases were successfully anonymised.",
                                       num) % str(num)
                             + " " + OK_TO_CONTINUE)
            c.url_ok = url_for(controller='/case_overview')
            del session['casebundle']
            return render ('/casebundle/dialogs/success_anonymize.mako')
        else:
            c.dialog_title = ungettext(u'Anonymise case?', u'Anonymise cases?',
                                       len(case_bundle.getCases()))
            c.dialog_text = (
                ungettext(u"Do you really want to anonymise the case and"
                          u" irrevocably lose its personal data? The case"
                          u" will not be available for editing anymore."
                          u" The anonymised case will still be considered"
                          u" in the analysis.%s",
                          u"Do you really want to anonymise the cases and"
                          u" irrevocably lose the personal data? The cases"
                          u" will not be available for editing anymore."
                          u" The anonymised cases will still be considered"
                          u" in the analysis.%s",
                          len(case_bundle.getCases()))
                % "\n".join(self._buildCaseList(case_bundle)))
            c.url_yes = url_for(controller='/case_bundle', action='anonymize',
                                confirmed=1)
            c.url_no = url_for(controller='/case_overview')
            return render('/casebundle/dialogs/confirm_anonymize.mako')

    # RESTORE
    @checkRole(('admin'))
    def restore(self, confirmed):
        confirmed = self._checkBool(confirmed)
        c.context = "../main.mako"
        case_bundle = session.get('casebundle')
        if confirmed == 1:
            num = case_bundle.restore()
            c.dialog_title = ungettext(u"Case restored!", u"Cases restored!",
                                       num)
            c.dialog_text = ungettext(u"%s was successfully restored.",
                                      u"%s were successfully restored.",
                                      num) % num + " " + OK_TO_CONTINUE
            c.url_ok = url_for(controller='/case_overview')
            del session['casebundle']
            return render ('/casemanagement/dialogs/success_restore.mako')
        else:
            c.dialog_title = ungettext(u'Restore case?', u'Restore cases?',
                                       len(case_bundle.getCases()))
            c.dialog_text = (
                ungettext(u'Do you really want to restore the following case?'
                          u' It will be available again for editing.%s',
                          u'Do you really want to restore the following cases?'
                          u' They will be available again for editing.%s',
                          len(case_bundle.getCases()))
                % "\n".join(self._buildCaseList(case_bundle)))
            c.url_yes = url_for(controller='/case_bundle', action='restore',
                                confirmed=1)
            c.url_no = url_for(controller='/case_overview')
            return render('/casemanagement/dialogs/confirm_restore.mako')

    # ORGANISATION
    @checkRole(('admin'))
    def set_editor(self):
        c.context = "../main.mako"
        case_bundle = session.get('casebundle')
        validator = SetBundleEditorForm()
        try:
            form_result = validator.to_python(request.params)
            num = case_bundle.setEditor(form_result.get('editor'))
            user = UserObject(form_result.get('editor'))
            c.dialog_title = _(u"Editor appointed!")
            c.dialog_text= (
                ungettext(u"%s case was successfully transferred to %s %s",
                          u"%s cases were successfully transferred to %s %s",
                          num) % (num, user.first_name, user.last_name)
                + " " + OK_TO_CONTINUE)
            c.url_ok = url_for(controller='/case_overview')
            del session['casebundle']
            return render('/casebundle/dialogs/success_editor.mako')
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            return render('/casebundle/editor.mako')

    @checkRole(('admin', 'cm'))
    def set_standin(self):
        c.context = "../main.mako"
        case_bundle = session.get('casebundle')
        validator = SetBundleStandinForm()
        try:
            form_result = validator.to_python(request.params)
            num = case_bundle.setStandin(form_result.get('standin'))
            c.dialog_title = _(u"Stand-in appointed!")
            c.dialog_text = (
                ungettext(u"%s case was successfully assigned to a substitute.",
                          u"%s cases were successfully assigned to a"
                          u" substitute.",
                          num) % num + " " + OK_TO_CONTINUE)
            c.url_ok = url_for(controller='/case_overview')
            del session['casebundle']
            return render('/casebundle/dialogs/success_standin.mako')
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            return render('/casebundle/standin.mako')

    # EXPORT
    def export(self, type="XML"):
        c.context = "../main.mako"
        case_bundle = session.get('casebundle')
        # only pages that are not invisible can be selected by the user.
        # For one thing, the user doesn't know about invisible pages and
        # they are always exported by libformed anyway.
        c.pages = [page for page in g.formedTree.findAllByClass(PageNode)
                   if not page.isInvisible()]
        data = {'exporttype': type,
                'exportanonym': '1',
                'exportselection': [i.getName() for i in c.pages]}
        c.dialog_title = ungettext(u'Export case?', u'Export cases?',
                                   len(case_bundle.getCases()))
        c.dialog_text = "\n".join(self._buildCaseList(case_bundle))
        # logbook export is only possible for XML exports and if the
        # logbook is actually activated in the application.
        c.allow_logbook_export = (g.mpuls_config.is_enabled('case-module',
                                                            'logbook')
                                  and type == "XML")
        form = render('/casemanagement/dialogs/confirm_export.mako')
        return formencode.htmlfill.render(form, defaults=data,
                                          auto_insert_errors=False,
                                          errors={})

    def exportAction(self):
        validator = ExportSelectorValidator()
        case_bundle = session.get('casebundle')
        form_result = validator.to_python(request.params)
        exporttype = form_result.get('exporttype')
        exportanonym = form_result.get('exportanonym')
        exportselection = form_result.get('exportselection')
        num, export = case_bundle.export(exporttype,
                                         exportanonym,
                                         exportselection)
        del session['casebundle']
        if exporttype == "XML":
            return self._download_data(export, "application/xml", "xml",
                                       content_encoding="UTF-8")
        elif exporttype == "XLS":
            return self._download_data(export, "application/msexcel", "xls")
        elif exporttype == "CSV":
            return self._download_data(export, "application/zip", "zip")

    def _download_data(self, data, mime_type, extension, **extra):
        disposition = 'attachment; filename="fallakten-export.%s"' % extension
        data = paste.fileapp.DataApp(data,
                                     content_type=mime_type,
                                     content_disposition=disposition,
                                     cache_control='must-revalidate',
                                     pragma='public',
                                     **extra)
        return data(request.environ, self.start_response)

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
