# -*- coding: utf-8 -*-

import logging

from pylons import app_globals as g

from mpulsweb.lib.translation import _

from formed.instance.tree import SetDataException

from mpulsweb.lib.security import checkRole
from mpulsweb.lib.base import BaseController, render, c, h, session
from mpulsweb.lib.anonymize import determineAnonymizedCaseEndPhase
from mpulsweb.model.phase import ConsistenceCheckException

from mpulsweb.controllers.case import \
     MARKANONYMIZE_SUCCESS, MARKANONYMIZE_SUCCESS_TEXT, MARKANONYMIZE_FAILED, \
     MARKANONYMIZE_FAILED_TEXT


HEADER = _("Set declaration of consent?")

ACCEPT_QUESTION = _("""You have stated that there is a declaration of consent from
the young adult to the acquisition, handling and storage of personal data.
Are you sure this is correct?""")

SET_EE_ACCEPT_SUCCESS_HEADER = _(u"Privacy statement set!")
SET_EE_ACCEPT_SUCCESS_TEXT = _(u'Please click Bitte klick "OK" to continue work on the case')

PRIVACY_DECLINE_DISALLOWED = _('Decline privacy statement not possible')

PRIVACY_DECLINE_DISALLOWED_TEXT = _('The case is in a non consistent state und it will not be possible to anonimise it after retracting the privacy statement. The reason for this is listed below:<p>%s</p>')

PRIVACY_DECLINE = _(u"Retract privacy statement?")
PRIVACY_DECLINE_TEXT = _(u"The case will be anonymised if you retract the privacy statement.""")


log = logging.getLogger(__name__)


class PrivacyController(BaseController):

    '''
    Controller to handle privacy related things within mpuls. This mainly
    includes accepting or declining the 'Childs declaration of consent' (known
    in WASKA/WASKO) or similar. Additionally it is possible to list available
    static statements.
    '''
    @checkRole('cm')
    def index(self):
        return self.listStatements()

    @checkRole('cm')
    def listStatements(self):
        """Render a list of available static statements related to privacy"""
        return render('privacy/list_statements.mako')

    @checkRole('cm')
    def printStatement(self, id):
        id = self._checkInt(id)
        case = self._loadCase(id)
        statement = case.getPrivacyStatement()
        c.content = statement.fillout()
        return render('privacy/default_statement.mako')

    @checkRole('cm')
    def set(self, id):
        c.targeturl = h.url_for(controller="/case", action="digest", id=id)
        c.ds_id = id
        return render('privacy/edit_statement.mako')

    @checkRole('cm')
    def accept(self, id, confirmed):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        case = self._loadCase(id)
        if confirmed:
            statement = case.getPrivacyStatement()
            statement.sign()
            # TODO: Think about refreshing the state of the privacy
            # statement in the statement model
            session_case = session.get('case')
            session_case.privacy_statement = True
            session.save()
            c.dialog_title = SET_EE_ACCEPT_SUCCESS_HEADER
            c.dialog_text = SET_EE_ACCEPT_SUCCESS_TEXT
            c.url_ok = h.url_for(controller="/case", action="digest",
                                 id=case.id)
            return render('privacy/dialogs/accept_success.mako')
        else:
            c.dialog_title = HEADER
            c.dialog_text = ACCEPT_QUESTION
            c.url_yes = h.url_for(controller="privacy", action="accept",
                                  id=id, confirmed="1")
            c.url_no = h.url_for(controller="/case", action="select",
                                 id=id, confirmed=0)
            return render('privacy/dialogs/confirm.mako')

    @checkRole('cm')
    def decline(self, id, confirmed=False):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        case = self._loadCase(id)
        try:
            case.check_anonymizeability()
        except ConsistenceCheckException, e:
            c.dialog_title = PRIVACY_DECLINE_DISALLOWED
            c.dialog_text = h.literal(PRIVACY_DECLINE_DISALLOWED_TEXT % e.value)
            c.url_ok = h.url_for(controller='/case', action='digest',
                                 id=case.id)
            return render('/casemanagement/dialogs/failed_markanonymize.mako')

        if confirmed:
            # Mark case for anonymizion
            try:
                case.make_anonymizable()
                case.getState().setState(4)
                c.dialog_title = MARKANONYMIZE_SUCCESS
                c.dialog_text = MARKANONYMIZE_SUCCESS_TEXT
                c.url_ok = h.url_for ('/case_overview')
                statement = case.getPrivacyStatement()
                statement.decline()
                return render('/privacy/dialogs/decline_success.mako')
            except SetDataException, e:
                errors = []
                for k, values in e.errors.iteritems():
                    for v in values:
                        errors.append(v.msg)
                c.dialog_title = MARKANONYMIZE_FAILED
                c.dialog_text = MARKANONYMIZE_FAILED_TEXT % "<br>".join(errors) 
                c.url_ok = h.url_for(controller='/case', action='select',
                                     id=case.id, confirmed=1)
                return render('/privacy/dialogs/decline_failed.mako')

        else:
            # Get description of the phase which will be set after anonymisation
            pdescriptions = g.mpuls_config.get('phases', 'description')[0]
            c.dialog_title = PRIVACY_DECLINE
            c.dialog_text = PRIVACY_DECLINE_TEXT
            c.url_yes = h.url_for(controller="privacy", action="decline",
                                  id=id, confirmed="1")
            c.url_no = h.url_for(controller="/case", action="select",
                                 id=id, confirmed=0)
            return render('privacy/dialogs/confirm.mako')

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8:
