# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
# Torsten Irlaender <torsten.irlaender@intevation.de>
#

import os
import time
import re
import random
import md5
import logging
from codecs import getdecoder
from types import ListType, TupleType

import psycopg2 as dbapi

from decorator import decorator

from webob.exc import HTTPUnauthorized

from pylons import request, config, g

from mpulsweb.lib.db import DB, db, enter, leave

# normally we would import session from mpulsweb.lib.base, but to avoid
# circular imports we import it from app_globals.
from mpulsweb.lib.app_globals import session


log = logging.getLogger(__name__)


check_re = re.compile(r"^[a-z_0-9]+$")

FETCH_USER_DATA_SQL = """\
SELECT id, vorname, nachname, passwort_aendern, aktiviert,
       rolle, gid, last_login, vertreter
FROM ka_benutzer_tbl_view
WHERE login = %(login)s
"""
UPDATE_LASTLOGIN_SQL = """SELECT set_last_logintime(%(user_id)s)"""
UNAUTHORIZED_DETAIL = \
"""Sie sind nicht authorisiert, das angeforderte Dokument anzuschauen.
"""

ASCII_DECODER = getdecoder("ascii")

log = logging.getLogger(__name__)


def slashSplit(s):
    out = [] 
    for p in s.split("/"):
        if p.find("=") == -1 and out:
            out[-1] += '/'+ p
        else:
            out.append(p)
    return out

def validPassword(s):
    try:
        ASCII_DECODER(s)
    except (UnicodeEncodeError, UnicodeDecodeError):
        return False
    return True

def checkLogin(user, password):
    dbname = get_unmapped_db_name()
    if (not password or not validPassword(password)
        or not dbname or not check_re.match(dbname)
        or not check_re.match(user)):
        return None

    # imported here, because mpulsweb.lib.helpers also imports
    # mpulsweb.lib.security which leads to circular imports.
    # ensure_unicode shouldn't be needed here anymore anyway when the
    # database returns Unicode objects directly.
    from mpulsweb.lib.helpers import ensure_unicode

    # Also imported here because mpulsweb.model.user imports
    # mpulsweb.lib.security, which leads to circular imports.
    from mpulsweb.model.user import SessionUser, SessionSuperAdmin

    log.info('Login: user "%s" in "%s"' % (user, dbname))
    try:
        db_parameters = get_db_parameters(user, password,
                                          config.get('mpuls.db.host'),
                                          config.get('mpuls.db.port'),
                                          config.get('mpuls.db.database'),
                                          config.get('mpuls.db.user'))
        mydb = DB(**db_parameters)
        try:
            enter(mydb)
            conn, cursor = None, None
            try:
                conn = db.getConnection()
                cursor = conn.cursor()
                login = db_parameters["user"]
                fields = {'login': login}
                cursor.execute(FETCH_USER_DATA_SQL, fields)
                try:
                    result = cursor.fetchone()
                    data = {}
                    data['id'] = result[0]
                    data['firstname'] = ensure_unicode(result[1])
                    data['lastname'] = ensure_unicode(result[2])
                    data['newpass'] = result[3]
                    data['activated'] = result[4]
                    data['role'] = ensure_unicode(result[5])
                    data['gid'] = result[6]
                    data['last_login'] = result[7]
                    data['standin'] = result[8]
                except:
                    log.exception("Error fetching user data")
                    data = None
            finally:
                db.recycleConnection(conn, cursor)

            try:
                if data:
                    userobject = SessionUser(login, password)
                    userobject.setData(data)
                elif user == 'adm':
                    userobject = SessionSuperAdmin(login, password)
                else:
                    return None

                # UPDATE last login time
                if data:
                    try:
                        db.execute(UPDATE_LASTLOGIN_SQL,
                                   {'user_id': data.get('id')})
                    except StandardError:
                        log.exception("Error setting last login time for"
                                      " user %r" % (data.get('id'),))

            except StandardError:
                log.exception("Error creating user Object")

            #RETURN userobject
            log.info('Login ok :)')
            return userobject
        finally:
            leave(mydb)
            mydb.closeConnections()
    except dbapi.OperationalError, err:
        # This is likely a login failure
        # XXX: Is there a way in DB Api 2.0 to determine
        # this database independent?
        log.exception("OperationalError from database"
                      " while checking user credentials")
    except:
        log.exception("Exception while checking user credentials")
    log.info('Login failed :(')
    return None

def generateID(obj=None):
    array = [time.time(), os.times(), random.random()]
    if not obj is None:
        array.append(id(obj))
    return md5.new(str(array)).hexdigest()

def subnet(addr):
    """poor man's subnet splitting"""
    x = addr.rsplit('.', 1) # IPv4
    if len(x) == 2:
        return x[0]
    x = addr.rsplit(':', 1) # IPv6
    if len(x) == 2:
        return x[0]
    return addr

def userIdentity():
    env = request.environ
    return "%s" % env.get('HTTP_USER_AGENT', '')


def get_dbname_from_http_header():
    """Return the KA-specific part of the DB-name set in HTTP-Header fields.

    This function expects the CN part of the Subject-DN from the
    SSL-client-certificate in either the MPULS_DBNAME or the
    SSL_CLIENT_S_DN_CN HTTP request header field.  The CN must consist
    of at least four whitespace separated words.  The KA-specific part
    of the database name is the fourth of those words.

    If the value cannot be determined, which usually means that neither
    of the header fields is set, the function returns None.
    """
    dbname = request.environ.get('MPULS_DBNAME')
    if not dbname:
        # MPULS_DBNAME not set or empty, fall back to older request
        # header
        log.debug("No database name defined in http-header MPULS_DBNAME,"
                 " falling back to SSL_CLIENT_S_DN_CN")
        dbname = request.environ.get('SSL_CLIENT_S_DN_CN')

    if not dbname:
        return None

    return dbname.split(' ')[3].lower()


def get_dbname_from_config():
    """Return the KA-specific part of the database name from the configuration.
    The return value is the value of the config entry 'mpuls.db.name'.
    """
    return config.get('mpuls.db.name')

def get_dbname_from_list():
    """Return the name of the database name from the saved db name in the
    session. The session var gets set in the login-method of the auth controller."""
    return session.get('DB_NAME_FROM_LIST')

def get_unmapped_db_name():
    """Return the KA-specific part of the database name. In first place it
    tries to get the name from a saved db name in a session var.  If this ther
    is no value the value is taken from HTTP-Headers containing information
    from the SSL-client-certificate using get_dbname_from_http_header().  If
    those headers are not set, the value is taken from the configuration using
    get_dbname_from_config().
    """
    dbname = get_dbname_from_list()
    if not dbname:
        log.debug("Could not determine database name from Request params,"
                 " falling back to HTTP header ")
        dbname = get_dbname_from_http_header()

    if not dbname:
        log.debug("Could not determine database name from HTTP header,"
                 " falling back to config file")
        dbname = get_dbname_from_config()

    if not dbname:
        log.debug("Could not determine database name from configuration file")
        raise RuntimeError("Could not determine database name")

    return dbname

def getDbName():
    """Like get_unmapped_db_name, but apply the dbmapping to the name"""
    dbname = get_unmapped_db_name()
    mapped_parameters = g.mpuls_dbmapping.get_mapping().get(dbname)
    if mapped_parameters is not None:
        dbname = mapped_parameters.get("dbagency", dbname)
    return dbname


def get_kaname_from_http_header():
    """Return the KA-name given by HTTP-Header fields.

    This function expects the Subject-DN from the SSL-client-certificate
    in either the MPULS_APPNAME or the SSL_CLIENT_S_DN HTTP request
    header field.  The return value is the value of the last OU
    component of that DN.  Subject DNs used for MPuls installations
    usually contain multuple OU fields with the last one describing the
    institution the user works for.

    If the value cannot be determined, which usually means that neither
    of the header fields is set, the function returns None.
    """

    subject_dn = request.environ.get('MPULS_APPNAME')
    if not subject_dn:
        # MPULS_APPNAME not set or empty, fall back to older request
        # header
        log.debug("HTTP-Header field MPULS_APPNAME not set or empty,"
                 " falling back to SSL_CLIENT_S_DN")
        subject_dn = request.environ.get('SSL_CLIENT_S_DN')
    if not subject_dn:
        return None

    for f in slashSplit(subject_dn)[::-1]:
        field = f.split('=')
        if field[0] == "OU":
            return field[1]

    log.debug("Subject DN found, but it did not contain an OU field")
    return None


def get_kaname_from_config():
    """Return the KA-name from the configuration"""
    return config.get('mpuls.app.name')


def getKAName():
    kaname = get_kaname_from_http_header()

    if not kaname:
        log.debug("Could not determine KA-name from HTTP header,"
                 " falling back to config file")
        kaname = get_kaname_from_config()

    if not kaname:
        log.debug("Could not determine KA-Name from configuration file")
        raise RuntimeError("Could not determine KA-name name")

    return kaname


def get_db_parameters(login, password, host, port, dbtemplate, usertemplate):
    """Determine database connection parameters from user-specific information.

    Parameters:

      login, password -- Login name and corresponding password

      host, port -- Default host and portname of the database cluster

      dbtemplate -- Default template string used to derive the actual
                    database name from dbagency

      usertemplate -- Default template string used to derive the actual
                      database username to use when logging into the
                      database, derived from dbagency and login name.

    The dbagency parameter is used to lookup agency specific settings in
    the configuration (via g.mpuls_dbmapping.get_mapping()) which can
    override any of the parameters host, port, dbtemplate, usertemplate
    and dbagency.

    The return value is a dictionary with the keys host, port, database,
    user and password.  It can be used directly as a dictionary for the
    keyword parameter of a DB-API connect() call.
    """
    dbagency = get_unmapped_db_name()
    mapping = g.mpuls_dbmapping.get_mapping()
    mapped_parameters = mapping.get(dbagency)
    if mapped_parameters:
        log.debug('mapped_parameters: %s' % mapped_parameters)
        dbagency = mapped_parameters.get('dbagency', dbagency)
        host = mapped_parameters.get('host', host)
        port = mapped_parameters.get('port', port)
        dbtemplate = mapped_parameters.get('dbtemplate', dbtemplate)
        usertemplate = mapped_parameters.get('usertemplate', usertemplate)

    return dict(host=host, port=port, database=dbtemplate % dbagency,
                user=usertemplate % (dbagency, login),
                password=password)


def checkRole(role):
    if type(role) in (ListType, TupleType):
        _role = role
    else:
        _role = (role,)

    def validate(func, self, *args, **kwargs):
        if not hasRole(_role):
            raise HTTPUnauthorized(detail=UNAUTHORIZED_DETAIL)
        try:
            return func(self, *args, **kwargs)
        except dbapi.OperationalError, err:
            # FIXME: An OperationalError at this point does not
            # necessarily indicate a permission problem, so converting
            # into it into a HTTPUnauthorized exception is not
            # necessarily the right thing, even though a permission
            # problem is a likely cause.  At the very least, log the
            # original exception because it's also likely to indicate a
            # problem with the database setup and discarding the
            # information of the original exception would make debugging
            # much harder.
            log.exception("Converting DB-API OperationalError into"
                          " an HTTP Unauthorized response")
            raise HTTPUnauthorized(detail=UNAUTHORIZED_DETAIL)

    return decorator(validate)

def hasRole(rolelist):
    old_role_names = {
        'cm_ka': 'cm',
        'admin_ka': 'admin',
        'pb_ka': 'pb',
        }
    try:
        user = session['USER_AUTHORIZED']
        conv_roles = []
        for r in rolelist:
            # Mapping of rolenames. The old rolenames are used all over the
            # application. So change them here.
            conv_roles.append(old_role_names.get(r, r))
        user_role = user.getPermissionRole()
        return user_role in conv_roles
    except KeyError:
        return False


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
